import * as React from "react";
import { createPortal } from "react-dom";
import { Rect } from "../Rect";
import { CLASSES } from "../Types";

/** @internal */
export interface IFloatingWindowProps {
    title: string;
    id: string;
    url: string;
    rect: Rect | null;
    onCloseWindow: (id: string) => void;
    onSetWindow: (id: string, window: Window) => void;
}

/** @internal */
export const FloatingWindow = (props: React.PropsWithChildren<IFloatingWindowProps>) => {
    const { id, url, rect, onCloseWindow, onSetWindow, children } = props;
    const popoutWindow = React.useRef<Window | null>(null);
    const timerId = React.useRef<any>(null);
    const [content, setContent] = React.useState<HTMLElement | undefined>(undefined);

    React.useLayoutEffect(() => {
        if (timerId.current) {
            clearTimeout(timerId.current);
        }
        let isMounted = true;
        const r = rect || new Rect(0, 0, 100, 100);

        popoutWindow.current = window.open(url, id, `left=${r.x},top=${r.y},width=${r.width},height=${r.height}`);
        if (popoutWindow.current !== null) {
            onSetWindow(id, popoutWindow.current);

            // Copiez les styles de la fenêtre principale vers la fenêtre flottante
            const mainDocument = document;
            const popoutDocument = popoutWindow.current.document;

            for (const styleSheet of mainDocument.styleSheets) {
                try {
                    const rules = styleSheet.cssRules || styleSheet.rules;
                    if (rules) {
                        const newStyleSheet = popoutDocument.createElement('style');
                        popoutDocument.head.appendChild(newStyleSheet);

                        for (const rule of rules) {
                            newStyleSheet.sheet?.insertRule(rule.cssText, newStyleSheet.sheet.rules.length);
                        }
                    }
                } catch (e) {
                    console.error(`Error copying styles: ${e}`);
                }
            }

            window.addEventListener("beforeunload", () => {
                if (popoutWindow.current) {
                    popoutWindow.current.close();
                    popoutWindow.current = null;
                }
            });

            popoutWindow.current.addEventListener("load", () => {
                if (isMounted) {
                    const popoutContent = popoutDocument.createElement("div");
                    popoutContent.className = CLASSES.FLEXLAYOUT__FLOATING_WINDOW_CONTENT;
                    popoutDocument.body.appendChild(popoutContent);

                    setContent(popoutContent);

                    popoutWindow.current!.addEventListener("beforeunload", () => {
                        onCloseWindow(id);
                    });
                }
            });
        } else {
            console.warn(`Unable to open window ${url}`);
            onCloseWindow(id);
        }

        return () => {
            isMounted = false;
            timerId.current = setTimeout(() => {
                if (popoutWindow.current) {
                    popoutWindow.current.close();
                    popoutWindow.current = null;
                }
            }, 0);
        };
    }, []);

    if (content !== undefined) {
        return createPortal(children, content!);
    } else {
        return null;
    }
};
